@@ -6,26 +6,31 @@ Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 This work can be distributed under the terms of the GNU GPLv3.
 '''
 
-from ..logging import logging, QuietError, LOG_ONCE # Ensure use of custom logger class
-from abc import abstractmethod, ABCMeta
-from functools import wraps
-import time
-import textwrap
 import hashlib
-import struct
 import hmac
-import random
 import inspect
-import ssl
+import logging
 import os
+import random
 import re
+import ssl
+import struct
+import textwrap
 import threading
+import time
+from abc import ABCMeta, abstractmethod
+from functools import wraps
+from io import BytesIO
+from typing import Any, BinaryIO, Dict, Optional
+
+from ..logging import LOG_ONCE, QuietError
 
 log = logging.getLogger(__name__)
 
+
 class RateTracker:
     '''
-    Maintain an average occurence rate for events over a configurable time
+    Maintain an average occurrence rate for events over a configurable time
     window. The rate is computed with one second resolution.
     '''
 
@@ -39,7 +44,7 @@ class RateTracker:
         self.lock = threading.Lock()
 
     def register(self, _not_really=False):
-        '''Register occurence of an event.
+        '''Register occurrence of an event.
 
         The keyword argument is for class-internal use only.
         '''
@@ -48,7 +53,7 @@ class RateTracker:
         bucket_count = len(self.buckets)
         now = int(time.monotonic())
 
-        elapsed =  min(now - self.last_update, bucket_count)
+        elapsed = min(now - self.last_update, bucket_count)
         for i in range(elapsed):
             buckets[(now - i) % bucket_count] = 0
 
@@ -60,7 +65,7 @@ class RateTracker:
             self.last_update = now
 
     def get_rate(self):
-        '''Return average rate of event occurance'''
+        '''Return average rate of event occurrence'''
 
         self.register(_not_really=True)
         return sum(self.buckets) / len(self.buckets)
@@ -78,15 +83,16 @@ class RateTracker:
 # multiple threads all have to retry after a long period of
 # inactivity.
 RETRY_TIMEOUT = 60 * 60 * 24
+
+
 def retry(method, _tracker=RateTracker(60)):
     '''Wrap *method* for retrying on some exceptions
 
-    If *method* raises an exception for which the instance's
-    `is_temp_failure(exc)` method is true, the *method* is called again at
-    increasing intervals. If this persists for more than `RETRY_TIMEOUT`
-    seconds, the most-recently caught exception is re-raised. If the
-    method defines a keyword parameter *is_retry*, then this parameter
-    will be set to True whenever the function is retried.
+    If *method* raises an exception for which the instance's `is_temp_failure(exc)` method is true,
+    the *method* is called again at increasing intervals. If this persists for more than
+    `RETRY_TIMEOUT` seconds, the most-recently caught exception is re-raised within a TimeoutError.
+    If the method defines a keyword parameter *is_retry*, then this parameter will be set to True
+    whenever the function is retried.
     '''
 
     if inspect.isgeneratorfunction(method):
@@ -103,28 +109,35 @@ def retry(method, _tracker=RateTracker(60)):
         retries = 0
         while True:
             if has_is_retry:
-                kw['is_retry'] = (retries > 0)
+                kw['is_retry'] = retries > 0
             try:
                 return method(*a, **kw)
             except Exception as exc:
                 # Access to protected member ok
-                #pylint: disable=W0212
+                # pylint: disable=W0212
                 if not self.is_temp_failure(exc):
                     raise
 
                 _tracker.register()
                 rate = _tracker.get_rate()
-                if  rate > 5:
-                    log.warning('Had to retry %d times over the last %d seconds, '
-                                'server or network problem?',
-                                rate * _tracker.window_length, _tracker.window_length)
+                if rate > 5:
+                    log.warning(
+                        'Had to retry %d times over the last %d seconds, '
+                        'server or network problem?',
+                        rate * _tracker.window_length,
+                        _tracker.window_length,
+                    )
                 else:
                     log.debug('Average retry rate: %.2f Hz', rate)
 
                 if waited > RETRY_TIMEOUT:
-                    log.error('%s.%s(*): Timeout exceeded, re-raising %r exception',
-                            self.__class__.__name__, method.__name__, exc)
-                    raise
+                    log.error(
+                        '%s.%s(*): Timeout exceeded, re-raising %r exception',
+                        self.__class__.__name__,
+                        method.__name__,
+                        exc,
+                    )
+                    raise TimeoutError() from exc
 
                 retries += 1
                 if retries <= 2:
@@ -134,9 +147,14 @@ def retry(method, _tracker=RateTracker(60)):
                 else:
                     log_fn = log.warning
 
-                log_fn('Encountered %s (%s), retrying %s.%s (attempt %d)...',
-                       type(exc).__name__, exc, self.__class__.__name__, method.__name__,
-                       retries)
+                log_fn(
+                    'Encountered %s (%s), retrying %s.%s (attempt %d)...',
+                    type(exc).__name__,
+                    exc,
+                    self.__class__.__name__,
+                    method.__name__,
+                    retries,
+                )
 
                 if hasattr(exc, 'retry_after') and exc.retry_after:
                     log.debug('retry_after is %.2f seconds', exc.retry_after)
@@ -146,16 +164,19 @@ def retry(method, _tracker=RateTracker(60)):
             # server with too many concurrent requests.
             time.sleep(interval * random.uniform(1, 1.5))
             waited += interval
-            interval = min(5*60, 2*interval)
+            interval = min(5 * 60, 2 * interval)
 
-    extend_docstring(wrapped,
-                     'This method has been wrapped and will automatically re-execute in '
-                     'increasing intervals for up to `s3ql.backends.common.RETRY_TIMEOUT` '
-                     'seconds if it raises an exception for which the instance\'s '
-                     '`is_temp_failure` method returns True.')
+    extend_docstring(
+        wrapped,
+        'This method has been wrapped and will automatically re-execute in '
+        'increasing intervals for up to `s3ql.backends.common.RETRY_TIMEOUT` '
+        'seconds if it raises an exception for which the instance\'s '
+        '`is_temp_failure` method returns True.',
+    )
 
     return wrapped
 
+
 def extend_docstring(fun, s):
     '''Append *s* to *fun*'s docstring with proper wrapping and indentation'''
 
@@ -170,19 +191,15 @@ def extend_docstring(fun, s):
             indent = min(indent, len(line) - len(stripped))
 
     indent_s = '\n' + ' ' * indent
-    fun.__doc__ += ''.join(indent_s + line
-                               for line in textwrap.wrap(s, width=80 - indent))
+    fun.__doc__ += ''.join(indent_s + line for line in textwrap.wrap(s, width=80 - indent))
     fun.__doc__ += '\n'
 
+
 class AbstractBackend(object, metaclass=ABCMeta):
     '''Functionality shared between all backends.
 
     Instances behave similarly to dicts. They can be iterated over and
     indexed into, but raise a separate set of exceptions.
-
-    The backend guarantees get after create consistency, i.e. a newly created
-    object will be immediately retrievable. Additional consistency guarantees
-    may or may not be available and can be queried for with instance methods.
     '''
 
     needs_login = True
@@ -202,7 +219,7 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def __iter__(self):
         return self.list()
 
-    def  __contains__(self, key):
+    def __contains__(self, key):
         return self.contains(key)
 
     def __enter__(self):
@@ -216,13 +233,6 @@ class AbstractBackend(object, metaclass=ABCMeta):
         for key in self.list():
             yield (key, self[key])
 
-    @property
-    @abstractmethod
-    def has_native_rename(self):
-        '''True if the backend has a native, atomic rename operation'''
-
-        pass
-
     @property
     def has_delete_multi(self):
         '''True if the backend supports `delete_multi`.'''
@@ -232,51 +242,41 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def reset(self):
         '''Reset backend
 
-        This resets the backend and ensures that it is ready to process
-        requests. In most cases, this method does nothing. However, if e.g. a
-        file handle returned by a previous call to `open_read` was not properly
-        closed (e.g. because an exception happened during reading), the `reset`
-        method will make sure that any underlying connection is properly closed.
-
-        Obviously, this method must not be called while any file handles
-        returned by the backend are still in use.
+        This resets the backend and ensures that it is ready to process requests. In most cases,
+        this method does nothing. However, if e.g. a previous request was not send completely or
+        response not read completely because of an exception, the `reset` method will make sure that
+        any underlying connection is properly closed.
         '''
 
         pass
 
-    @retry
-    def perform_read(self, fn, key):
-        '''Read object data using *fn*, retry on temporary failure
+    @abstractmethod
+    def readinto_fh(self, key: str, fh: BinaryIO):
+        '''Transfer data stored under *key* into *fh*, return metadata.
 
-        Open object for reading, call `fn(fh)` and close object. If a temporary
-        error (as defined by `is_temp_failure`) occurs during opening, closing
-        or execution of *fn*, the operation is retried.
+        The data will be inserted at the current offset. If a temporary error (as defined by
+        `is_temp_failure`) occurs, the operation is retried.
         '''
+        pass
 
-        fh = self.open_read(key)
-        try:
-            res = fn(fh)
-        except Exception as exc:
-            # If this is a temporary failure, we now that the call will be
-            # retried, so we don't need to warn that the object was not read
-            # completely.
-            fh.close(checksum_warning=not self.is_temp_failure(exc))
-            raise
-        else:
-            fh.close()
-            return res
+    @abstractmethod
+    def write_fh(
+        self,
+        key: str,
+        fh: BinaryIO,
+        metadata: Optional[Dict[str, Any]] = None,
+        len_: Optional[int] = None,
+    ):
+        '''Upload *len_* bytes from *fh* under *key*.
 
-    @retry
-    def perform_write(self, fn, key, metadata=None, is_compressed=False):
-        '''Read object data using *fn*, retry on temporary failure
+        The data will be read at the current offset. If *len_* is None, reads until the
+        end of the file.
 
-        Open object for writing, call `fn(fh)` and close object. If a temporary
-        error (as defined by `is_temp_failure`) occurs during opening, closing
-        or execution of *fn*, the operation is retried.
+        If a temporary error (as defined by `is_temp_failure`) occurs, the operation is
+        retried.  Returns the size of the resulting storage object (which may be less due
+        to compression)
         '''
-
-        with self.open_write(key, metadata, is_compressed) as fh:
-            return fn(fh)
+        pass
 
     def fetch(self, key):
         """Return data stored under `key`.
@@ -286,11 +286,9 @@ class AbstractBackend(object, metaclass=ABCMeta):
         ``backend.fetch(key)[0]``.
         """
 
-        def do_read(fh):
-            data = fh.read()
-            return (data, fh.metadata)
-
-        return self.perform_read(do_read, key)
+        fh = BytesIO()
+        metadata = self.readinto_fh(key, fh)
+        return (fh.getvalue(), metadata)
 
     def store(self, key, val, metadata=None):
         """Store data under `key`.
@@ -304,7 +302,7 @@ class AbstractBackend(object, metaclass=ABCMeta):
         equivalent to ``backend.store(key, val)``.
         """
 
-        self.perform_write(lambda fh: fh.write(val), key, metadata)
+        return self.write_fh(key, BytesIO(val), metadata)
 
     @abstractmethod
     def is_temp_failure(self, exc):
@@ -336,38 +334,6 @@ class AbstractBackend(object, metaclass=ABCMeta):
         '''Return size of object stored under *key*'''
         pass
 
-    @abstractmethod
-    def open_read(self, key):
-        """Open object for reading
-
-        Return a file-like object. Data can be read using the `read`
-        method. Metadata is returned in the file-like object's *metadata*
-        attribute and can be modified by the caller at will. The object must be
-        closed explicitly.
-        """
-
-        pass
-
-    @abstractmethod
-    def open_write(self, key, metadata=None, is_compressed=False):
-        """Open object for writing
-
-        `metadata` can be mapping with additional attributes to store with the
-        object. Keys have to be of type `str`, values have to be of elementary
-        type (`str`, `bytes`, `int`, `float` or `bool`).
-
-        Returns a file- like object. The object must be closed closed
-        explicitly. After closing, the *get_obj_size* may be used to retrieve
-        the size of the stored object (which may differ from the size of the
-        written data).
-
-        The *is_compressed* parameter indicates that the caller is going to
-        write compressed data, and may be used to avoid recompression by the
-        backend.
-        """
-
-        pass
-
     def contains(self, key):
         '''Check if `key` is in backend'''
 
@@ -379,30 +345,26 @@ class AbstractBackend(object, metaclass=ABCMeta):
             return True
 
     @abstractmethod
-    def delete(self, key, force=False):
+    def delete(self, key):
         """Delete object stored under `key`
 
-        ``backend.delete(key)`` can also be written as ``del backend[key]``.  If
-        `force` is true, do not return an error if the key does not exist. Note,
-        however, that even if *force* is False, it is not guaranteed that an
-        attempt to delete a non-existing object will raise an error.
+        Attempts to delete non-existing objects will silently succeed.
         """
         pass
 
-    def delete_multi(self, keys, force=False):
+    def delete_multi(self, keys):
         """Delete objects stored under `keys`
 
         Deleted objects are removed from the *keys* list, so that the caller can
         determine which objects have not yet been processed if an exception is
         occurs.
 
-        If *force* is True, attempts to delete non-existing objects will
-        succeed. Note, however, that even if *force* is False, it is not
-        guaranteed that an attempt to delete a non-existing object will raise an
-        error.
+        Attempts to delete non-existing objects will silently succeed.
         """
 
-        raise NotImplemented()
+        while keys:
+            self.delete(keys[-1])
+            keys.pop()
 
     @abstractmethod
     def list(self, prefix=''):
@@ -412,42 +374,6 @@ class AbstractBackend(object, metaclass=ABCMeta):
         '''
         pass
 
-    @abstractmethod
-    def copy(self, src, dest, metadata=None):
-        """Copy data stored under key `src` to key `dest`
-
-        If `dest` already exists, it will be overwritten. If *metadata* is
-        `None` metadata will be copied from the source as well, otherwise
-        *metadata* becomes the metadata for the new object.
-
-        Copying will be done on the remote side without retrieving object data.
-        """
-
-        pass
-
-    @abstractmethod
-    def update_meta(self, key, metadata):
-        """Replace metadata of *key* with *metadata*
-
-        `metadata` must be a mapping with keys of type `str`, and values of an
-        elementary type (`str`, `bytes`, `int`, `float` or `bool`).
-        """
-
-        pass
-
-    def rename(self, src, dest, metadata=None):
-        """Rename key `src` to `dest`
-
-        If `dest` already exists, it will be overwritten. If *metadata* is
-        `None` metadata will be preserved, otherwise *metadata* becomes the
-        metadata for the renamed object.
-
-        Rename done remotely without retrieving object data.
-        """
-
-        self.copy(src, dest, metadata)
-        self.delete(src)
-
     def close(self):
         '''Close any opened resources
 
@@ -460,6 +386,7 @@ class AbstractBackend(object, metaclass=ABCMeta):
 
         pass
 
+
 class NoSuchObject(Exception):
     '''Raised if the requested object does not exist in the backend'''
 
@@ -470,6 +397,7 @@ class NoSuchObject(Exception):
     def __str__(self):
         return 'Backend does not have anything stored under key %r' % self.key
 
+
 class DanglingStorageURLError(Exception):
     '''Raised if the backend can't store data at the given location'''
 
@@ -484,6 +412,7 @@ class DanglingStorageURLError(Exception):
         else:
             return self.msg
 
+
 class AuthorizationError(Exception):
     '''Raised if the credentials don't give access to the requested backend'''
 
@@ -494,6 +423,7 @@ class AuthorizationError(Exception):
     def __str__(self):
         return 'Access denied. Server said: %s' % self.msg
 
+
 class AuthenticationError(Exception):
     '''Raised if the credentials are invalid'''
 
@@ -504,6 +434,7 @@ class AuthenticationError(Exception):
     def __str__(self):
         return 'Access denied. Server said: %s' % self.msg
 
+
 class CorruptedObjectError(Exception):
     """
     Raised if a storage object is corrupted.
@@ -519,12 +450,13 @@ class CorruptedObjectError(Exception):
     def __str__(self):
         return self.str
 
+
 def get_ssl_context(path):
     '''Construct SSLContext object'''
 
     # Best practice according to http://docs.python.org/3/library/ssl.html#protocol-versions
-    context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-    context.options |= ssl.OP_NO_SSLv2
+    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+    context.minimum_version = ssl.TLSVersion.TLSv1_2
     context.verify_mode = ssl.CERT_REQUIRED
 
     if path is None:
@@ -539,6 +471,7 @@ def get_ssl_context(path):
 
     return context
 
+
 def get_proxy(ssl):
     '''Read system proxy settings
 
@@ -556,12 +489,16 @@ def get_proxy(ssl):
         proxy = os.environ[proxy_env]
         hit = re.match(r'^(https?://)?([a-zA-Z0-9.-]+)(:[0-9]+)?/?$', proxy)
         if not hit:
-            raise QuietError('Unable to parse proxy setting %s=%r' %
-                             (proxy_env, proxy), exitcode=13)
+            raise QuietError(
+                'Unable to parse proxy setting %s=%r' % (proxy_env, proxy), exitcode=13
+            )
 
         if hit.group(1) == 'https://':
-            log.warning('HTTPS connection to proxy is probably pointless and not supported, '
-                        'will use standard HTTP', extra=LOG_ONCE)
+            log.warning(
+                'HTTPS connection to proxy is probably pointless and not supported, '
+                'will use standard HTTP',
+                extra=LOG_ONCE,
+            )
 
         if hit.group(3):
             proxy_port = int(hit.group(3)[1:])
@@ -569,14 +506,14 @@ def get_proxy(ssl):
             proxy_port = 80
 
         proxy_host = hit.group(2)
-        log.info('Using proxy %s:%d', proxy_host, proxy_port,
-                 extra=LOG_ONCE)
+        log.info('Using proxy %s:%d', proxy_host, proxy_port, extra=LOG_ONCE)
         proxy = (proxy_host, proxy_port)
     else:
         proxy = None
 
     return proxy
 
+
 def checksum_basic_mapping(metadata, key=None):
     '''Compute checksum for mapping of elementary types
 
