@@ -6,22 +6,27 @@ Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 This work can be distributed under the terms of the GNU GPLv3.
 '''
 
-from .logging import logging, setup_logging, QuietError
-from .common import assert_fs_owner
-from .parse_args import ArgumentParser
-import pyfuse3
+import logging
 import os
 import stat
 import sys
 import textwrap
 
+import pyfuse3
+
+from .common import assert_fs_owner
+from .logging import QuietError, setup_logging, setup_warnings
+from .parse_args import ArgumentParser
+
 log = logging.getLogger(__name__)
 
+
 def parse_args(args):
     '''Parse command line'''
 
     parser = ArgumentParser(
-        description=textwrap.dedent('''\
+        description=textwrap.dedent(
+            '''\
         Replicates the contents of the directory <source> in the
         directory <target>. <source> has to be an existing directory and
         <target>  must not exist. Both directories have to be within
@@ -30,28 +35,30 @@ def parse_args(args):
         The replication will not take any additional space. Only if one
         of directories is modified later on, the modified data will take
         additional storage space.
-        '''))
+        '''
+        )
+    )
 
     parser.add_log()
     parser.add_debug()
     parser.add_quiet()
     parser.add_version()
 
-    parser.add_argument('source', help='source directory',
-                        type=(lambda x: x.rstrip('/')))
-    parser.add_argument('target', help='target directory',
-                        type=(lambda x: x.rstrip('/')))
+    parser.add_argument('source', help='source directory', type=(lambda x: x.rstrip('/')))
+    parser.add_argument('target', help='target directory', type=(lambda x: x.rstrip('/')))
 
     options = parser.parse_args(args)
 
     return options
 
+
 def main(args=None):
     '''Efficiently copy a directory tree'''
 
     if args is None:
         args = sys.argv[1:]
 
+    setup_warnings()
     options = parse_args(args)
     setup_logging(options)
 
@@ -90,11 +97,10 @@ def main(args=None):
 
     # Ensure the inode of the target folder stays in the kernel dentry cache
     # (We invalidate it during the copy)
-    with os.scandir(options.target) as it:
-
+    with os.scandir(options.target):
         fstat_t = os.stat(options.target)
-        pyfuse3.setxattr(ctrlfile, 'copy',
-                         ('(%d, %d)' % (fstat_s.st_ino, fstat_t.st_ino)).encode())
+        pyfuse3.setxattr(ctrlfile, 'copy', ('(%d, %d)' % (fstat_s.st_ino, fstat_t.st_ino)).encode())
+
 
 if __name__ == '__main__':
     main(sys.argv[1:])
