@@ -8,36 +8,40 @@ This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 if __name__ == '__main__':
-    import pytest
     import sys
+
+    import pytest
+
     sys.exit(pytest.main([__file__] + sys.argv[1:]))
 
-from argparse import Namespace
-from common import safe_sleep
-from contextlib import contextmanager
-from pytest_checklogs import assert_logs
-from s3ql.backends import local
-from s3ql.backends.common import AbstractBackend
-from s3ql.backends.pool import BackendPool
-from s3ql.block_cache import BlockCache, QuitSentinel
-from s3ql.common import time_ns
-from s3ql.database import Connection
-from s3ql.metadata import create_tables
-from s3ql.mkfs import init_tables
-from queue import Full as QueueFull
-from unittest.mock import patch
 import logging
 import os
-import pytest
 import queue
 import shutil
 import stat
 import tempfile
 import threading
+from argparse import Namespace
+from contextlib import contextmanager
+from queue import Full as QueueFull
+
+import pytest
 import trio
+from common import safe_sleep
+from pytest_checklogs import assert_logs
+
+from s3ql.backends import local
+from s3ql.backends.common import AbstractBackend
+from s3ql.backends.comprenc import ComprencBackend
+from s3ql.backends.pool import BackendPool
+from s3ql.block_cache import BlockCache, QuitSentinel
+from s3ql.common import time_ns
+from s3ql.database import Connection, create_tables
+from s3ql.mkfs import init_tables
 
 log = logging.getLogger(__name__)
 
+
 # A dummy removal queue to monkeypatch around the need for removal threads
 class DummyQueue:
     def __init__(self, cache):
@@ -67,59 +71,87 @@ class DummyQueue:
     def qsize(self):
         return 0
 
+
 def random_data(len_):
     with open("/dev/urandom", "rb") as fh:
         return fh.read(len_)
 
-@pytest.yield_fixture
+
+@pytest.fixture
 async def ctx():
-        ctx = Namespace()
-        ctx.backend_dir = tempfile.mkdtemp(prefix='s3ql-backend-')
-        ctx.backend_pool = BackendPool(lambda: local.Backend(
-            Namespace(storage_url='local://' + ctx.backend_dir)))
+    ctx = Namespace()
+    ctx.backend_dir = tempfile.mkdtemp(prefix='s3ql-backend-')
+    ctx.backend_pool = BackendPool(
+        lambda: ComprencBackend(
+            b'foobar',
+            ('zlib', 6),
+            local.Backend(Namespace(storage_url='local://' + ctx.backend_dir)),
+        )
+    )
 
-        ctx.cachedir = tempfile.mkdtemp(prefix='s3ql-cache-')
-        ctx.max_obj_size = 1024
+    ctx.cachedir = tempfile.mkdtemp(prefix='s3ql-cache-')
+    ctx.max_obj_size = 1024
 
-        # Destructors are not guaranteed to run, and we can't unlink
-        # the file immediately because apsw refers to it by name.
-        # Therefore, we unlink the file manually in tearDown()
-        ctx.dbfile = tempfile.NamedTemporaryFile(delete=False)
-        ctx.db = Connection(ctx.dbfile.name)
-        create_tables(ctx.db)
-        init_tables(ctx.db)
+    # Destructors are not guaranteed to run, and we can't unlink
+    # the file immediately because apsw refers to it by name.
+    # Therefore, we unlink the file manually in tearDown()
+    ctx.dbfile = tempfile.NamedTemporaryFile(delete=False)
 
-        # Create an inode we can work with
-        ctx.inode = 42
-        now_ns = time_ns()
-        ctx.db.execute("INSERT INTO inodes (id,mode,uid,gid,mtime_ns,atime_ns,ctime_ns,refcount,size) "
-                        "VALUES (?,?,?,?,?,?,?,?,?)",
-                        (ctx.inode, stat.S_IFREG | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-                         | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH,
-                         os.getuid(), os.getgid(), now_ns, now_ns, now_ns, 1, 32))
+    ctx.db = Connection(ctx.dbfile.name)
+    create_tables(ctx.db)
+    init_tables(ctx.db)
 
-        cache = BlockCache(ctx.backend_pool, ctx.db, ctx.cachedir + "/cache",
-                           ctx.max_obj_size * 100)
-        cache.trio_token = trio.lowlevel.current_trio_token()
-        ctx.cache = cache
+    # Create an inode we can work with
+    ctx.inode = 42
+    now_ns = time_ns()
+    ctx.db.execute(
+        "INSERT INTO inodes (id,mode,uid,gid,mtime_ns,atime_ns,ctime_ns,refcount,size) "
+        "VALUES (?,?,?,?,?,?,?,?,?)",
+        (
+            ctx.inode,
+            stat.S_IFREG
+            | stat.S_IRUSR
+            | stat.S_IWUSR
+            | stat.S_IXUSR
+            | stat.S_IRGRP
+            | stat.S_IXGRP
+            | stat.S_IROTH
+            | stat.S_IXOTH,
+            os.getuid(),
+            os.getgid(),
+            now_ns,
+            now_ns,
+            now_ns,
+            1,
+            32,
+        ),
+    )
 
-        # Monkeypatch around the need for removal and upload threads
-        cache.to_remove = DummyQueue(cache)
-        class DummyChannel:
-            async def send(self, arg):
-                await trio.to_thread.run_sync(cache._do_upload, *arg)
-        cache.to_upload = (DummyChannel(), None)
+    cache = BlockCache(ctx.backend_pool, ctx.db, ctx.cachedir + "/cache", ctx.max_obj_size * 100)
+    cache.trio_token = trio.lowlevel.current_trio_token()
+    ctx.cache = cache
+
+    # Monkeypatch around the need for removal and upload threads
+    cache.to_remove = DummyQueue(cache)
+
+    class DummyChannel:
+        async def send(self, arg):
+            await trio.to_thread.run_sync(cache._do_upload, *arg)
+
+    cache.to_upload = (DummyChannel(), None)
+
+    try:
+        yield ctx
+    finally:
+        ctx.cache.backend_pool = ctx.backend_pool
+        if ctx.cache.destroy is not None:
+            await ctx.cache.destroy()
+        shutil.rmtree(ctx.cachedir)
+        shutil.rmtree(ctx.backend_dir)
+        ctx.db.close()
+        ctx.dbfile.close()
+        os.unlink(ctx.dbfile.name)
 
-        try:
-            yield ctx
-        finally:
-            ctx.cache.backend_pool = ctx.backend_pool
-            if ctx.cache.destroy is not None:
-                await ctx.cache.destroy()
-            shutil.rmtree(ctx.cachedir)
-            shutil.rmtree(ctx.backend_dir)
-            ctx.dbfile.close()
-            os.unlink(ctx.dbfile.name)
 
 async def test_thread_hang(ctx):
     # Make sure that we don't deadlock if uploads threads or removal
@@ -129,18 +161,21 @@ async def test_thread_hang(ctx):
     # in other threads
     upload_exc = False
     removal_exc = False
+
     def _upload_loop(*a, fn=ctx.cache._upload_loop):
         try:
             return fn(*a)
         except NotADirectoryError:
             nonlocal upload_exc
             upload_exc = True
+
     def _removal_loop_multi(*a, fn=ctx.cache._removal_loop_multi):
         try:
             return fn(*a)
         except NotADirectoryError:
             nonlocal removal_exc
             removal_exc = True
+
     ctx.cache._upload_loop = _upload_loop
     ctx.cache._removal_loop_multi = _removal_loop_multi
 
@@ -166,8 +201,9 @@ async def test_thread_hang(ctx):
 
     try:
         # Try to clean-up (implicitly calls expire)
-        with assert_logs('Unable to flush cache, no upload threads left alive',
-                         level=logging.ERROR, count=1):
+        with assert_logs(
+            'Unable to flush cache, no upload threads left alive', level=logging.ERROR, count=1
+        ):
             await ctx.cache.destroy(keep_cache=True)
         assert upload_exc
         assert removal_exc
@@ -181,6 +217,7 @@ async def test_thread_hang(ctx):
         await ctx.cache.remove(ctx.inode, 1)
         ctx.cache.destroy = None
 
+
 async def test_get(ctx):
     inode = ctx.inode
     blockno = 11
@@ -202,6 +239,7 @@ async def test_get(ctx):
         fh.seek(0)
         assert data == fh.read(len(data))
 
+
 @pytest.mark.trio
 async def test_expire(ctx):
     inode = ctx.inode
@@ -237,6 +275,7 @@ async def test_expire(ctx):
         else:
             assert (inode, i) in ctx.cache.cache
 
+
 async def test_upload(ctx):
     inode = ctx.inode
     datalen = int(0.1 * ctx.cache.cache.max_size)
@@ -308,6 +347,7 @@ async def test_upload(ctx):
     await ctx.cache.drop()
     ctx.cache.backend_pool.verify()
 
+
 async def test_remove_referenced(ctx):
     inode = ctx.inode
     datalen = int(0.1 * ctx.cache.cache.max_size)
@@ -329,6 +369,7 @@ async def test_remove_referenced(ctx):
     await ctx.cache.remove(inode, blockno1)
     ctx.cache.backend_pool.verify()
 
+
 async def test_remove_cache(ctx):
     inode = ctx.inode
     data1 = random_data(int(0.4 * ctx.max_obj_size))
@@ -340,7 +381,8 @@ async def test_remove_cache(ctx):
     await ctx.cache.remove(inode, 1)
     async with ctx.cache.get(inode, 1) as fh:
         fh.seek(0)
-        assert fh.read(42) ==  b''
+        assert fh.read(42) == b''
+
 
 async def test_upload_race(ctx):
     inode = ctx.inode
@@ -356,6 +398,7 @@ async def test_upload_race(ctx):
     # Try to upload it, may happen if CommitThread is interrupted
     await ctx.cache.upload_if_dirty(fh)
 
+
 async def test_expire_race(ctx):
     # Create element
     inode = ctx.inode
@@ -385,7 +428,7 @@ async def test_expire_race(ctx):
 
         assert len(ctx.cache.cache) == 0
     finally:
-            await ctx.cache.mlock.release(inode, blockno, noerror=True)
+        await ctx.cache.mlock.release(inode, blockno, noerror=True)
 
 
 async def test_parallel_expire(ctx):
@@ -416,7 +459,7 @@ async def test_parallel_expire(ctx):
 
         assert len(ctx.cache.cache) == 4
     finally:
-            await ctx.cache.mlock.release(inode, 0, noerror=True)
+        await ctx.cache.mlock.release(inode, 0, noerror=True)
 
 
 async def test_remove_cache_db(ctx):
@@ -438,6 +481,7 @@ async def test_remove_cache_db(ctx):
         fh.seek(0)
         assert fh.read(42) == b''
 
+
 async def test_remove_db(ctx):
     inode = ctx.inode
     data1 = random_data(int(0.4 * ctx.max_obj_size))
@@ -454,42 +498,7 @@ async def test_remove_db(ctx):
     ctx.cache.backend_pool.verify()
     async with ctx.cache.get(inode, 1) as fh:
         fh.seek(0)
-        assert fh.read(42) ==  b''
-
-# Disabled, still needs to be ported to trio.
-async def XXtest_issue_241(ctx):
-
-    inode = ctx.inode
-
-    # Create block
-    async with ctx.cache.get(inode, 0) as fh:
-        fh.write(random_data(500))
-
-    # "Fill" cache
-    ctx.cache.cache.max_entries = 0
-
-    # Mock locking to reproduce race condition
-    mlock = MockMultiLock(ctx.cache.mlock)
-    async with trio.open_nursery() as nursery:
-        async with trio.open_nursery() as nursery2:
-            with patch.object(ctx.cache, 'mlock', mlock):
-                # Start expiration, will block on lock
-                await nursery.start(ctx.cache.expire)
-
-                # Remove the object while the expiration thread waits
-                # for it to become available.
-                await nursery2.start(ctx.cache.remove, inode, 0, 1)
-
-                mlock.yield_to(thread2)
-
-        # Create a new object for the same block
-        async with ctx.cache.get(inode, 0) as fh:
-            fh.write(random_data(500))
-
-        # Continue first expiration run
-        mlock.yield_to(thread1, block=False)
-        thread1.join_and_raise(timeout=10)
-        assert not thread1.is_alive()
+        assert fh.read(42) == b''
 
 
 class MockMultiLock:
@@ -549,8 +558,6 @@ class MockMultiLock:
 
 
 class MockBackendPool(AbstractBackend):
-    has_native_rename = False
-
     def __init__(self, backend_pool, no_read=0, no_write=0, no_del=0):
         super().__init__()
         self.no_read = no_read
@@ -565,9 +572,9 @@ class MockBackendPool(AbstractBackend):
 
     def verify(self):
         if self.no_read != 0:
-            raise RuntimeError('Got too few open_read calls')
+            raise RuntimeError('Got too few readinto_fh calls')
         if self.no_write != 0:
-            raise RuntimeError('Got too few open_write calls')
+            raise RuntimeError('Got too few write_fh calls')
         if self.no_del != 0:
             raise RuntimeError('Got too few delete calls')
 
@@ -581,19 +588,18 @@ class MockBackendPool(AbstractBackend):
     def lookup(self, key):
         return self.backend.lookup(key)
 
-    def open_read(self, key):
+    def readinto_fh(self, key, fh):
         self.no_read -= 1
         if self.no_read < 0:
-            raise RuntimeError('Got too many open_read calls')
+            raise RuntimeError('Got too many readinto_fh calls')
+        return self.backend.readinto_fh(key, fh)
 
-        return self.backend.open_read(key)
-
-    def open_write(self, key, metadata=None, is_compressed=False):
+    def write_fh(self, key, fh, metadata=None, dont_compress=False, len_=None):
         self.no_write -= 1
         if self.no_write < 0:
-            raise RuntimeError('Got too many open_write calls')
+            raise RuntimeError('Got too many write_fh calls')
 
-        return self.backend.open_write(key, metadata, is_compressed)
+        return self.backend.write_fh(key, fh, metadata, dont_compress=dont_compress, len_=len_)
 
     def is_temp_failure(self, exc):
         return self.backend.is_temp_failure(exc)
@@ -601,12 +607,12 @@ class MockBackendPool(AbstractBackend):
     def contains(self, key):
         return self.backend.contains(key)
 
-    def delete(self, key, force=False):
+    def delete(self, key):
         self.no_del -= 1
         if self.no_del < 0:
             raise RuntimeError('Got too many delete calls')
 
-        return self.backend.delete(key, force)
+        return self.backend.delete(key)
 
     def list(self, prefix=''):
         '''List keys in backend
@@ -629,6 +635,7 @@ class MockBackendPool(AbstractBackend):
 
         return self.backend.get_size(key)
 
+
 async def start_flush(cache, inode, block=None):
     """Upload data for `inode`
 
@@ -648,7 +655,7 @@ async def start_flush(cache, inode, block=None):
         await cache.upload_if_dirty(el)
 
 
-class MockLock():
+class MockLock:
     def __call__(self):
         return self
 
