@@ -8,24 +8,25 @@ This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 if __name__ == '__main__':
-    import pytest
     import sys
+
+    import pytest
+
     sys.exit(pytest.main([__file__] + sys.argv[1:]))
 
-from common import populate_dir, skip_without_rsync, get_remote_test_info, NoTestSection
-from s3ql import backends
-from s3ql.database import Connection
 import shutil
 import subprocess
-from subprocess import check_output, CalledProcessError
-import t4_fuse
 import tempfile
+from subprocess import CalledProcessError, check_output
+
 import pytest
-import os
-from s3ql.common import _escape
+import t4_fuse
+from common import NoTestSection, get_remote_test_info, populate_dir, skip_without_rsync
+
+from s3ql import backends
+
 
 class TestFull(t4_fuse.TestFuse):
-
     def populate_dir(self, path):
         populate_dir(path)
 
@@ -35,20 +36,9 @@ class TestFull(t4_fuse.TestFuse):
         ref_dir = tempfile.mkdtemp(prefix='s3ql-ref-')
         try:
             self.populate_dir(ref_dir)
-
-            # Copy source data
             self.mkfs()
-
-            # Force 64bit inodes
-            cachepath = os.path.join(self.cache_dir, _escape(self.storage_url))
-            db = Connection(cachepath + '.db')
-            db.execute('UPDATE sqlite_sequence SET seq=? WHERE name=?',
-                       (2 ** 36 + 10, 'inodes'))
-            db.close()
-
             self.mount()
-            subprocess.check_call(['rsync', '-aHAX', ref_dir + '/',
-                                   self.mnt_dir + '/'])
+            subprocess.check_call(['rsync', '-aHAX', ref_dir + '/', self.mnt_dir + '/'])
             self.umount()
             self.fsck()
 
@@ -58,9 +48,19 @@ class TestFull(t4_fuse.TestFuse):
             self.fsck()
             self.mount()
             try:
-                out = check_output(['rsync', '-anciHAX', '--delete', '--exclude', '/lost+found',
-                                    ref_dir + '/', self.mnt_dir + '/'], universal_newlines=True,
-                                  stderr=subprocess.STDOUT)
+                out = check_output(
+                    [
+                        'rsync',
+                        '-anciHAX',
+                        '--delete',
+                        '--exclude',
+                        '/lost+found',
+                        ref_dir + '/',
+                        self.mnt_dir + '/',
+                    ],
+                    universal_newlines=True,
+                    stderr=subprocess.STDOUT,
+                )
             except CalledProcessError as exc:
                 pytest.fail('rsync failed with ' + exc.output)
             if out:
@@ -82,8 +82,7 @@ class RemoteTest:
     def setup_method(self, method, name):
         super().setup_method(method)
         try:
-            (backend_login, backend_pw,
-             self.storage_url) = get_remote_test_info(name)
+            (backend_login, backend_pw, self.storage_url) = get_remote_test_info(name)
         except NoTestSection as exc:
             super().teardown_method(method)
             pytest.skip(exc.reason)
@@ -91,15 +90,17 @@ class RemoteTest:
         self.backend_passphrase = backend_pw
 
     def populate_dir(self, path):
-        populate_dir(path, entries=50, size=5*1024*1024)
+        populate_dir(path, entries=50, size=5 * 1024 * 1024)
 
     def teardown_method(self, method):
         super().teardown_method(method)
 
-        proc = subprocess.Popen(self.s3ql_cmd_argv('s3qladm') +
-                                [ '--quiet', '--authfile', '/dev/null',
-                                  'clear', self.storage_url ],
-                                stdin=subprocess.PIPE, universal_newlines=True)
+        proc = subprocess.Popen(
+            self.s3ql_cmd_argv('s3qladm')
+            + ['--quiet', '--authfile', '/dev/null', 'clear', self.storage_url],
+            stdin=subprocess.PIPE,
+            universal_newlines=True,
+        )
         if self.backend_login is not None:
             print(self.backend_login, file=proc.stdin)
             print(self.backend_passphrase, file=proc.stdin)
@@ -113,9 +114,11 @@ class RemoteTest:
 for backend_name in backends.prefix_map:
     if backend_name == 'local':
         continue
+
     def setup_method(self, method, backend_name=backend_name):
         RemoteTest.setup_method(self, method, backend_name + '-test')
+
     test_class_name = 'TestFull' + backend_name
-    globals()[test_class_name] = type(test_class_name,
-                                      (RemoteTest, TestFull),
-                                      { 'setup_method': setup_method })
+    globals()[test_class_name] = type(
+        test_class_name, (RemoteTest, TestFull), {'setup_method': setup_method}
+    )
